@@ -1,5 +1,4 @@
-// Copyright (C) 2018   John Donoghue   <john.donoghue@ieee.org>
-// Copyright (C) 2016   John Donoghue   <john.donoghue@ieee.org>
+// Copyright (C) 2016-2019   John Donoghue   <john.donoghue@ieee.org>
 // Copyright (C) 2013   Stefan Mahr     <dac922@gmx.de>
 // Copyright (C) 2012   Andrius Sutas   <andrius.sutas@gmail.com>
 //
@@ -55,6 +54,23 @@
 #  define socklen_t int
 #endif
 
+static struct timeval
+to_timeval(int ms)
+{
+  struct timeval tv;
+  if(ms <= 0)
+    {
+      tv.tv_usec = 0;
+      tv.tv_sec = 0;
+    }
+    else
+    {
+      tv.tv_usec = (ms % 1000) * 1000;
+      tv.tv_sec = ms/1000;;
+    }
+  return tv;
+}
+
 static std::string 
 to_ip_str (const sockaddr_in *in)
 {
@@ -92,7 +108,7 @@ lookup_addr (const std::string &ip, sockaddr_in *in)
 DEFINE_OV_TYPEID_FUNCTIONS_AND_DATA (octave_udp, "octave_udp", "octave_udp");
 
 octave_udp::octave_udp (void)
-: fd(-1), timeout(-1), name("")
+: fd(-1), timeout(-1), name(""), fieldnames(9)
 {
   static bool type_registered = false;
 
@@ -101,6 +117,89 @@ octave_udp::octave_udp (void)
       type_registered = true;
       register_type ();
     }
+  fieldnames[0] = "type";
+  fieldnames[1] = "name";
+  fieldnames[2] = "remoteport";
+  fieldnames[3] = "remotehost";
+  fieldnames[4] = "localport";
+  fieldnames[5] = "status";
+  fieldnames[6] = "timeout";
+  fieldnames[7] = "bytesavailable";
+  fieldnames[8] = "localhost";
+}
+
+octave_value_list
+octave_udp::subsref (const std::string& type, const std::list<octave_value_list>& idx, int nargout)
+{
+  octave_value_list retval;
+  int skip = 1;
+
+  switch (type[0])
+    {
+    default:
+      error ("octave_udp object cannot be indexed with %c", type[0]);
+      break;
+    case '.':
+      {
+        octave_value_list ovl;
+        // inc ref count as assign this to octave_value
+        count++; 
+        ovl (0) = octave_value (this);
+        ovl (1) = (idx.front ()) (0);
+        retval = OCTAVE__FEVAL (std::string ("__udp_properties__"), ovl, 1);
+      }
+      break;
+    }
+
+  if (idx.size () > 1 && type.length () > 1)
+    retval = retval (0).next_subsref (nargout, type, idx, skip);
+
+  return retval;
+}
+
+octave_value
+octave_udp::subsasgn (const std::string& type, const std::list<octave_value_list>& idx, const octave_value& rhs)
+{
+  octave_value retval;
+
+  switch (type[0])
+    {
+    default:
+      error ("octave_udp object cannot be indexed with %c", type[0]);
+      break;
+    case '.':
+      if (type.length () == 1)
+        {
+          octave_value_list ovl;
+          // inc ref count as assign this to octave_value
+          count++; 
+          ovl (0) = octave_value (this);
+          ovl (1) = (idx.front ()) (0);
+          ovl (2) = rhs;
+          OCTAVE__FEVAL (std::string ("__udp_properties__"), ovl, 1);
+          count++;
+          retval = octave_value (this);
+        }
+      else if (type.length () > 1 && type[1] == '.')
+        {
+          // pass along any further assignments
+          octave_value_list u = subsref (type.substr (0, 1), idx, 1);
+          if (u.length () > 0)
+            {
+              std::list<octave_value_list> next_idx (idx);
+              next_idx.erase (next_idx.begin ());
+              u (0).subsasgn(type.substr (1), next_idx, rhs);
+              count++;
+              retval = octave_value (this);
+            } 
+        }
+      else
+        {
+          error ("octave_udp invalid index");
+        }
+
+    }
+  return retval;
 }
 
 int
@@ -219,7 +318,7 @@ octave_udp::get_bytesavailable () const
 }
 
 int
-octave_udp::read (uint8_t *buf, unsigned int len, int readtimeout)
+octave_udp::read (uint8_t *buf, unsigned int len, double readtimeout)
 {
   struct sockaddr_in addr;
   socklen_t addrlen = sizeof (addr);
@@ -241,19 +340,7 @@ octave_udp::read (uint8_t *buf, unsigned int len, int readtimeout)
     {
       OCTAVE_QUIT;
 
-      if (readtimeout < 0) 
-        {
-          tv.tv_sec = 1;
-          tv.tv_usec = 0;
-        }
-      else
-        {
-          tv.tv_sec = 0;
-          if (readtimeout > 1000)
-            tv.tv_usec = 1000 * 1000;
-          else
-            tv.tv_usec = readtimeout * 1000;
-        }
+      tv = to_timeval((readtimeout < 0 || readtimeout > 1000) ? 1000 : readtimeout);
 
       FD_ZERO (&readfds);
       FD_SET (get_fd (), &readfds);
@@ -325,7 +412,7 @@ octave_udp::write (uint8_t *buf, unsigned int len)
 }
 
 int
-octave_udp::set_timeout (int newtimeout)
+octave_udp::set_timeout (double newtimeout)
 {
   if (get_fd () < 0)
     {
@@ -391,6 +478,13 @@ octave_udp::get_local_port (void) const
   return ntohs (local_addr.sin_port);
 }
 
+std::string
+octave_udp::get_local_addr (void) const
+{
+  return  to_ip_str (&local_addr);
+}
+
+
 std::string
 octave_udp::set_name (const std::string &n)
 {
