@@ -1,4 +1,4 @@
-// Copyright (C) 2016   John Donoghue   <john.donoghue@ieee.org>
+// Copyright (C) 2016-2019 John Donoghue   <john.donoghue@ieee.org>
 // Copyright (C) 2013   Stefan Mahr     <dac922@gmx.de>
 // Copyright (C) 2012   Andrius Sutas   <andrius.sutas@gmail.com>
 //
@@ -38,7 +38,7 @@ public:
   int write (const std::string &str);
   int write (uint8_t *buf, unsigned int len);
 
-  int read (uint8_t *buf, unsigned int len, int readtimeout);
+  int read (uint8_t *buf, unsigned int len, double readtimeout);
 
   int open (const std::string &address, int port, int localport);
 
@@ -63,7 +63,30 @@ public:
   // Properties
   bool is_constant (void) const { return true; }
   bool is_defined (void) const { return true; }
+  // < 4.4
   bool is_object (void) const { return true; }
+  // 4.4 +
+  bool isobject (void) const { return true; }
+
+  // required to use subsasn
+  string_vector map_keys (void) const { return fieldnames; }
+  dim_vector dims (void) const { static dim_vector dv(1, 1); return dv; }
+
+  // use single copy of each udp socket
+  octave_base_value * unique_clone (void) { return this; }
+
+ /**
+  * overloaded methods to get properties
+  */
+  octave_value_list subsref (const std::string& type, const std::list<octave_value_list>& idx, int nargout);
+
+  octave_value subsref (const std::string& type, const std::list<octave_value_list>& idx)
+  {
+    octave_value_list retval = subsref (type, idx, 1);
+    return (retval.length () > 0 ? retval(0) : octave_value ());
+  }
+
+  octave_value subsasgn (const std::string& type, const std::list<octave_value_list>& idx, const octave_value& rhs);
 
   std::string get_name (void) const { return name; }
   std::string set_name (const std::string &);
@@ -76,22 +99,25 @@ public:
 
   int get_local_port (void) const;
 
+  std::string get_local_addr (void) const;
+
   bool is_open(void) const;
 
   std::string get_type (void) const { return "udp"; }
 
   std::string get_status (void) const;
 
-  int set_timeout (int);
-  int get_timeout (void) const { return timeout; }
+  int set_timeout (double);
+  double get_timeout (void) const { return timeout; }
 
   int flush(int mode);
 private:
   int fd;
-  int timeout;
+  double timeout;
   std::string name;
   sockaddr_in remote_addr;
   sockaddr_in local_addr;
+  string_vector fieldnames;
 
   DECLARE_OV_TYPEID_FUNCTIONS_AND_DATA
 };
